function [proj] = projection_explicit(bewley, N);

lambda          = bewley.lambda;
decis           = bewley.decis;
deciskpv        = 1+mod(decis-1,bewley.knum);
decisef         = 1+(decis - deciskpv)/bewley.knum;
deciskp         = reshape(deciskpv,bewley.knum,bewley.nx*bewley.ne);
decisef         = reshape(decisef,bewley.knum,bewley.nx*bewley.ne);
kgrid           = bewley.kgrid;
wage            = bewley.wage;
beta            = bewley.bet;
Transg          = bewley.Transg;
Transyk         = bewley.Transyk;
v               = bewley.v;
r               = bewley.r;
phil            = bewley.phil;
xsil            = bewley.xsil;
Ny              = bewley.nx;
Ne              = bewley.ne;



Uc              = @(C,l) (C - (l.^(1+1/phil))/(xsil*(1+1/phil))).^(-bewley.sigma);

if bewley.sigma == 1
    U           = @(C,l) log(C - (l.^(1+1/phil))/(xsil*(1+1/phil)));
else
    U           = @(C,l) (((C - (l.^(1+1/phil))/(xsil*(1+1/phil)))).^(1-bewley.sigma))/(1-bewley.sigma);
end

%%%% rehaping the allocation in a matric form and computing welfare
lambdam         = reshape(lambda,bewley.knum,bewley.nx*bewley.ne);
amat            = kron(kgrid,ones(1,bewley.nx*bewley.ne));%beginning of period wealth
apmat           = kgrid(deciskp); % end-of-period wealth


%%% Reaching the exact financial market eq.
Ktot            = bewley.meanL*((bewley.r +bewley.delta)/bewley.alpha)^(1/(bewley.alpha-1));
coefa           = Ktot/(sum(sum(lambdam.*apmat)));


lle              = (bewley.xsil*(1-bewley.tau)*bewley.ex*bewley.wage).^bewley.phil;
wmat            = kron(ones(bewley.knum,1),[bewley.b*wage*(bewley.ex.*lle);(1-bewley.tau)*wage*(bewley.ex.*lle)]'); % lab income
cmat            = (1+r)*amat + wmat - apmat;

Ctot            = sum(sum(lambdam.*cmat));
aend            = apmat(:);
abeg            = amat(:);

nuvm            = bewley.nuvm;
nuvm(find(nuvm<0))= 0;


vmat            = reshape(v,bewley.knum,bewley.nx*bewley.ne);   
sizem           = sum(lambdam);

Nidio           = bewley.nx*bewley.ne;
Nbin              = Nidio^N;

proj.lambda = zeros(Nbin,1);
ResMat      = zeros(length(bewley.lambda),Nbin);

for i = 1:Nbin
    lambdav = bewley.lambda;
    v   = convertBasis10E(i,Nidio-1,N);
    for j = 1:N
        state = v(N+1-j);               % first item of v is the most recent
        Transg_state = 0*bewley.Transg;
        Transg_state((state-1)*bewley.knum+1:state*bewley.knum,:) = bewley.Transg((state-1)*bewley.knum+1:state*bewley.knum,:);
        lambdav  = Transg_state*lambdav;
    end;
    ResMat(:,i) = lambdav;
end;


% Taking average (Not per  capita)
proj.lambdam = zeros(Nbin,1);
proj.c       = zeros(Nbin,1);
proj.a       = zeros(Nbin,1);
proj.ap      = zeros(Nbin,1);
proj.w       = zeros(Nbin,1);
proj.nu      = zeros(Nbin,1);
proj.v       = zeros(Nbin,1);
proj.uc      = zeros(Nbin,1);
proj.ul      = zeros(Nbin,1);
proj.er      = zeros(Nbin,1);
proj.Euc     = zeros(Nbin,1);

% vector of marginal utilities
Lspv         = [kron(bewley.b*lle',ones(bewley.knum,1)),kron(lle',ones(bewley.knum,1))];
Ucvi         = Uc(cmat(:),Lspv(:));
Ucvimat      = reshape(Ucvi,bewley.knum,bewley.nx*bewley.ne); 
Ul           = U(cmat(:),Lspv(:));
Ulmat        = reshape(Ul,bewley.knum,bewley.nx*bewley.ne); 
errorE       = Ucvi - beta*(1+r)*Transg'*Ucvi - nuvm(:); % constructing vector of errors of Euler equations
errorEmat   = reshape(errorE,bewley.knum,bewley.nx*bewley.ne); 


% average value by bin Not per capita!
for i      = 1:Nbin
    lambdav = ResMat(:,i);
    proj.lambda(i) = sum(lambdav);
    proj.c(i)       = lambdav' * cmat(:);
    proj.a(i)       = lambdav'* amat(:);
    proj.ap(i)      = lambdav' * apmat(:);
    proj.w(i)       = lambdav' * wmat(:);
    proj.nu(i)      = lambdav' * nuvm(:);
    proj.v(i)       = lambdav' * vmat(:);
    proj.uc(i)      = lambdav' * Ucvimat(:);
    proj.ul(i)      = lambdav' * Ulmat(:);
    proj.er(i)      = lambdav' * errorEmat(:);          % level of utility

    lambdaf = bewley.Transg*lambdav;                    % next period distribution condition on beeing in bin i
    proj.Euc(i) = lambdaf'*Ucvimat(:);                  % expected marginal utility
        
end;

%%%%constructing the transition matrices on the projection
Transp               = zeros(Nbin,Nbin); % general transition matrix AGENTS
for i        = 1:Nbin
    v        = convertBasis10E(i,Nidio-1,N);
    statei   = v(1);
     if statei <= Nidio/2
        ei   = 1;
        b    = 0.5;
    else
        ei   = 2;
        b    = 1;
    end;
    yi       = statei - (ei-1)*Ny;
    Lsp(i)   = b*bewley.ex(yi)*lle;
    
    
    for j      = 1:Nbin
         vf    = convertBasis10E(j,Nidio-1,N);
        statej = vf(1);
         if statej <= Nidio/2
           ej  = 1;
         else
            ej = 2;
         end;
         yj    = statej - (ej-1)*Ny;
         if vf(2:end)  ==   v(1:end-1) % if possible continuation 
              Transp (j,i)   =  bewley.pie(ei,ej)*bewley.piex(yi,yj); % from i to j
         end
       end;       
       
    end;  

Transpa = Transp;
    
%check   proj.lambda' - Transp*proj.lambda'=0
%check   proj.a  - Transp*proj.ap


Transpu = 0*Transp;
for i = 1:Nbin
    lambdav     = ResMat(:,i);
    v           = convertBasis10E(i,Nidio-1,N);
    for j       = 1:Nidio
        vf      = 0*v;
        vf(2:length(v)) = v(1:end-1);
        vf(1)   = j;
        hf      = convertBasisE10(vf,Nidio-1,N);
        
        state   = j;
        Transg_state    = 0*bewley.Transg;
        Transg_state((state-1)*bewley.knum+1:state*bewley.knum,:) = bewley.Transg((state-1)*bewley.knum+1:state*bewley.knum,:);
        lambdaf         = Transg_state*lambdav;
        Transpu(i,hf)   = lambdaf'*Ucvimat(:); % i toward hj, contribution of h to expected marginal utility of i
    end;
end;

MatEU       = kron(proj.Euc,ones(length(Transpu),1));
Transpu     = Transpu./kron(proj.uc',ones(length(Transpu),1));

% check proj.Euc - Transpu*proj.uc



Ucv          = 0; % vector of marginal utility (before xsiE)
if bewley.ne>1
    Lsp      = [bewley.b*kron(lle',ones(Nbin/2,1)),kron(lle',ones(Nbin/2,1))];
else
      Lsp    = kron(lle',ones(Nbin,1));
end;
Lsp         = Lsp(:);
cv          = proj.c(:) ./proj.lambda;
Ucv         = Uc(cv,Lsp);
xsiE        = (proj.uc(:)./proj.lambda(:))./Ucv;




%check      proj.uc - beta*(1+r)*Transpu*proj.uc  - proj.nu(:) - proj.er(:) = 0;
%check      xsiE.*proj.lambda.*Ucv - beta*(1+r)* Transpu*(xsiE.*proj.lambda.*Ucv)  - proj.nu(:) -  proj.er(:)

% To have the same definition of Transpu in projection_implict
%Transpu         = (diag(proj.lambda)*Transpu*diag(1./proj.lambda));

%check      xsiE.*proj.lambda.*Ucv - beta*(1+r)* (diag(1./proj.lambda)*Transpu*diag(proj.lambda)) *(xsiE.*proj.lambda.*Ucv)  - proj.nu(:) -  proj.er(:)

% check    proj.a(:) - proj.Transpa*proj.ap(:)   = 0
% check     proj.a(:) - proj.Transpa*proj.ap(:)     = 0 j in t-1 i in t
%check      xsiE.*proj.lambda.*Ucv = beta*(1+r)*Transpu*(xsiE.*proj.lambda.*Ucv)  + proj.nu(:) + proj.er(:);
%check      xsiE.*proj.lambda.*Ucv = beta*(1+r)* (diag(1./proj.lambda)Transpu*diag(proj.lambda)) *(xsiE.*proj.lambda.*Ucv)  + proj.nu(:) + proj.er(:);

%%% projecting  welfare (eta)
eta                = (proj.ul(:)./proj.lambda)./U(cv,Lsp);

%%% recap
proj.method        = 1; % 1 for explicit 2 for implicit
proj.lambdam       = proj.lambdam ; % matrix form
proj.lambda        = proj.lambda ; % vector form
proj.c             = proj.c;
proj.a             = proj.a;
proj.ap            = proj.ap;
proj.w             = proj.w;
proj.nu            = proj.nu;
proj.v             = proj.v;
proj.uc            = proj.uc;
proj.ul            = proj.ul;
proj.er            = proj.er; 
proj.Euc           = proj.Euc;

%%%  projection result
proj.Nbin           = Nbin;
proj.xsiE           = xsiE;
proj.Transp         = Transp;    % backward agents
proj.N              = N;
proj.Ncons          = 1;
proj.Transpa        = Transpa;   % backwar wealth
proj.eta            = eta;
proj.ls             = lle;
proj.Transpu        = Transpu; % forward Euler  per capita forward




%%% to follow parameters
proj.r              = bewley.r;
proj.beta           = bewley.bet;
proj.alpha          = bewley.alpha;
proj.delta          = bewley.delta;
proj.wage           = bewley.wage;
proj.tau            = bewley.tau;
proj.phi            = bewley.b;
proj.sigma          = bewley.sigma;
proj.nx             = bewley.nx;     
proj.ne             = bewley.ne;
proj.ex             = bewley.ex;
proj.coefa          = coefa;
proj.v              = proj.v(:)./proj.lambda; %per capita
proj.cv             = cv;
proj.Ucv            = Ucv;
proj.Lsp            = Lsp;

proj.vareta         = var(proj.lambda.*proj.eta);
proj.varxsiE        = var(proj.lambda.*proj.xsiE);
proj.meaneta        = mean(proj.eta);
proj.meanxsiE       = mean(proj.xsiE);

proj.sdtdE          = std(proj.xsiE);
proj.sdtdW          = std(proj.eta);



