function [proj] = projection_implict(bewley, N);


% Construct implicit projection with an intended number of bin N

lambda          = bewley.lambda;
decis           = bewley.decis;
deciskpv        = 1+mod(decis-1,bewley.knum);
decisef         = 1+(decis - deciskpv)/bewley.knum;
deciskp         = reshape(deciskpv,bewley.knum,bewley.nx*bewley.ne);
decisef         = reshape(decisef,bewley.knum,bewley.nx*bewley.ne);
kgrid           = bewley.kgrid;
wage            = bewley.wage;
beta            = bewley.bet;
Transg          = bewley.Transg;
Transyk         = bewley.Transyk;
v               = bewley.v;
r               = bewley.r;
phil            = bewley.phil;
xsil            = bewley.xsil;


Uc              = @(C,l) (C - (l.^(1+1/phil))/(xsil*(1+1/phil))).^(-bewley.sigma);

if bewley.sigma == 1
    U           = @(C,l) log(C - (l.^(1+1/phil))/(xsil*(1+1/phil)));
else
     U          = @(C,l) (((C - (l.^(1+1/phil))/(xsil*(1+1/phil)))).^(1-bewley.sigma))/(1-bewley.sigma);
end

%%%% rehaping the allocation in a matric form and computing welfare
lambdam         = reshape(lambda,bewley.knum,bewley.nx*bewley.ne);
amat            = kron(kgrid,ones(1,bewley.nx*bewley.ne));%beginning of period wealth
apmat           = kgrid(deciskp); % end-of-period wealth


%%% Reaching the exact financial market eq.
Ktot            = bewley.meanL*((bewley.r +bewley.delta)/bewley.alpha)^(1/(bewley.alpha-1));
coefa           = Ktot/(sum(sum(lambdam.*apmat)));

lle             = (bewley.xsil*(1-bewley.tau)*bewley.ex*bewley.wage).^bewley.phil;
wmat            = kron(ones(bewley.knum,1),[bewley.b*wage*(bewley.ex.*lle);(1-bewley.tau)*wage*(bewley.ex.*lle)]'); % lab income
cmat            = (1+r)*amat + wmat - apmat;

Ctot            = sum(sum(lambdam.*cmat));
aend            = apmat(:);
abeg            = amat(:);


nuvm            = bewley.nuvm;
nuvm(find(nuvm<0))= 0;


vmat            = reshape(v,bewley.knum,bewley.nx*bewley.ne);   

sizem           = sum(lambdam);
cumlab          = cumsum(lambdam);
cumlab          = cumlab./cumlab(end,:);

%constructing the indexes for the various bin by type (proj.ind)
% Taking care of CC agents
inc           = 1/N;
proj.lambda   = zeros(N,bewley.nx*bewley.ne);
proj.ind      = zeros(N,bewley.nx*bewley.ne);
proj.ind(1,:) = 1;
Ncons         = 0;
for ty        = 1:bewley.nx*bewley.ne
    ind       = max(find(deciskp(:,ty)==1));
        if ind > 0 % credit constrained          
            proj.ind(2,ty)       = ind+1;
            Ncons                = Ncons+1;
        end;
end;
 
indt         = ones(N,1);                                   % find index in the wealth distribution
DD           = cumsum(sum(lambdam')');
i            = 2;
for j        = 1:N
   index     = max(find(DD<inc*j));
   if index>indt(i-1)
     indt(i) = index;
     i       = i+1;
   end;  
end;


indc                                = max(max(proj.ind));   % last bin of unconstrained
proj.ind(2,find(proj.ind(2,:)==0))  = indc;                 % define second bin of unconstrained agents
idmin                               = min(find(indt>indc));
Mat                                 = kron(ones(1,bewley.nx*bewley.ne),indt(idmin:i-1,:));
proj.ind(3:2+size(Mat,1),:)         = Mat;
proj.ind(3+size(Mat,1),:)           = bewley.knum+1;
Nbin                                = 3+size(Mat,1)-1;


% Taking average (Not per  capita)
proj.lambdam = zeros(Nbin,bewley.nx*bewley.ne);
proj.c       = zeros(Nbin,bewley.nx*bewley.ne);
proj.a       = zeros(Nbin,bewley.nx*bewley.ne);
proj.ap      = zeros(Nbin,bewley.nx*bewley.ne);
proj.w       = zeros(Nbin,bewley.nx*bewley.ne);
proj.ef      = zeros(Nbin,bewley.nx*bewley.ne);
proj.nu      = zeros(Nbin,bewley.nx*bewley.ne);
proj.v       = zeros(Nbin,bewley.nx*bewley.ne);
proj.uc      = zeros(Nbin,bewley.nx*bewley.ne);
proj.ul      = zeros(Nbin,bewley.nx*bewley.ne);
proj.er      = zeros(Nbin,bewley.nx*bewley.ne);


% vector of marginal utilities
Lspv         = [kron(bewley.b*lle',ones(bewley.knum,1)),kron(lle',ones(bewley.knum,1))];
Ucvi         = Uc(cmat(:),Lspv(:));
Ucvimat      = reshape(Ucvi,bewley.knum,bewley.nx*bewley.ne); 
Ul           = U(cmat(:),Lspv(:));
Ulmat        = reshape(Ul,bewley.knum,bewley.nx*bewley.ne); 
errorE       = Ucvi - beta*(1+r)*Transg'*Ucvi - nuvm(:); % constructing vector of errors of Euler equations
errorEmat    = reshape(errorE,bewley.knum,bewley.nx*bewley.ne); 



% average value by bin Not per capita!
for ty      = 1:bewley.nx*bewley.ne
    for j   = 1:Nbin
        proj.lambdam(j,ty) = sum(lambdam(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty));
        proj.c(j,ty)       = lambdam(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty)' * cmat(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty);
        proj.a(j,ty)       = lambdam(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty)' * amat(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty);
        proj.ap(j,ty)      = lambdam(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty)' * apmat(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty);
        proj.w(j,ty)       = lambdam(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty)' * wmat(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty);
        proj.nu(j,ty)      = lambdam(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty)' * nuvm(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty);
        proj.v(j,ty)       = lambdam(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty)' * vmat(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty);
        proj.uc(j,ty)      = lambdam(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty)' * Ucvimat(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty);
        proj.ul(j,ty)      = lambdam(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty)' * Ulmat(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty); % level of utility
        proj.er(j,ty)      = lambdam(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty)' * errorEmat(proj.ind(j,ty):proj.ind(j+1,ty)-1,ty); % level of utility
    end;
end;

%%%%constructing the transition matrices on the projection

Transp               = 0; % general transition matrix AGENTS
Transpa              = 0; % general transition matrix WEALTH


proj.lambda     = 0;
for ty       = 1:bewley.nx*bewley.ne
    ind      = (ty-1)*bewley.knum;
    for i    = 1:Nbin
        vect                                             = zeros(length(Transg),1);
        vect(ind+ proj.ind(i,ty):ind+proj.ind(i+1,ty)-1) = lambda(ind+ proj.ind(i,ty):ind+proj.ind(i+1,ty)-1);
        proj.lambda((ty-1)*(Nbin)+i,1)                   = sum(vect);
        vectg                                            = Transg*vect;
        abin                                             = aend(ind+ proj.ind(i,ty):ind+proj.ind(i+1,ty)-1)'*lambda(ind+ proj.ind(i,ty):ind+proj.ind(i+1,ty)-1);
        for ty2           = 1:bewley.nx*bewley.ne     
            ind2          = (ty2-1)*bewley.knum; 
            for jbin      = 1:Nbin
                ind2      = (ty2-1)*bewley.knum;
                vect2                                                       = zeros(length(Transg),1);
                vecta                                                       = zeros(length(Transg),1);
                vect2(ind2+ proj.ind(jbin,ty2):ind2+proj.ind(jbin+1,ty2)-1) = 1;
                vecta(ind2+ proj.ind(jbin,ty2):ind2+proj.ind(jbin+1,ty2)-1) = abeg(ind2+ proj.ind(jbin,ty2):ind2+proj.ind(jbin+1,ty2)-1);
                if proj.lambda((ty-1)*(Nbin)+i)>0
                     Transp((ty2-1)*Nbin+jbin,(ty-1)*Nbin+i)                = vect2'*vectg/proj.lambda((ty-1)*(Nbin)+i); % fraction switching (in%)                       
                else
                     Transp((ty2-1)*(Nbin)+jbin,(ty-1)*(Nbin)+i)            = 0;                   
                     Transp((ty-1)*(Nbin)+i,(ty-1)*(Nbin)+i)                = 1;                    
                end
                if abin>0
                     Transpa((ty2-1)*(Nbin)+jbin,(ty-1)*(Nbin)+i)           = vecta'*vectg/abin; % fraction of wealth switching (in%)                      
                else
                     Transpa((ty2-1)*(Nbin)+jbin,(ty-1)*(Nbin)+i)           = 0;
                     Transpa((ty-1)*(Nbin)+i,(ty-1)*(Nbin)+i)               = 1;
                     
                end
            end
        end;       
       
    end;   
end;



% Euler equation

for ty       = 1:bewley.nx*bewley.ne
    ind      = (ty-1)*bewley.knum;
    for i    = 1:Nbin
        vect                                             = zeros(length(Transg),1);        
        vect(ind+ proj.ind(i,ty):ind+proj.ind(i+1,ty)-1) = lambda(ind+ proj.ind(i,ty):ind+proj.ind(i+1,ty)-1);
        vectg                                            = Transg*vect;
        
        for ty2           = 1:bewley.nx*bewley.ne     
            ind2          = (ty2-1)*bewley.knum; 
            for jbin      = 1:Nbin
                ind2      = (ty2-1)*bewley.knum;
         
                vectE                                                       = zeros(length(Transg),1);
                vect2                                                       = zeros(length(Transg),1);
                vect2(ind2+ proj.ind(jbin,ty2):ind2+proj.ind(jbin+1,ty2)-1) = lambda(ind2+ proj.ind(jbin,ty2):ind2+proj.ind(jbin+1,ty2)-1);
                vectE(ind2+ proj.ind(jbin,ty2):ind2+proj.ind(jbin+1,ty2)-1) = Ucvi(ind2+ proj.ind(jbin,ty2):ind2+proj.ind(jbin+1,ty2)-1);
                Transpu((ty-1)*(Nbin)+i,(ty2-1)*(Nbin)+jbin)                = vectE'*vectg/(vectE'*vect2);                       
            end
        end;       
       
    end;   
end;


%ZZ = Uch - beta*(1+r)*Transpu*Uch  - proj.nu(:) - proj.er(:);

Ntot                = length(Transp);

% projecting Euler equations (xsi)
Ucv                      = 0; % vector of marginal utility (before xsiE)
Lsp                      = [bewley.b*kron(lle',ones(Nbin,1)),kron(lle',ones(Nbin,1))];
Lsp                      = Lsp(:);
cv                       = proj.c(:) ./proj.lambda;
Ucv                      = Uc(cv,Lsp);
Uch = proj.uc(:);
xsiE                     = (Uch./proj.lambdam(:))./Ucv;



ZZ2                      = xsiE.*proj.lambda.*Ucv - beta*(1+r)*Transpu*(xsiE.*proj.lambda.*Ucv)  - proj.nu(:) - proj.er(:);

  


% check    proj.a(:) - proj.Transpa*proj.ap(:)   = 0
% check    proj.a(:) - proj.Transpa*proj.ap(:)     = 0 j in t-1 i in t
% check    xsiE.*proj.lambda.*Ucv = beta*(1+r)*Transpu*(xsiE.*proj.lambda.*Ucv)  + proj.nu(:) + proj.er(:);
% check    xsiE.*proj.lambda.*Ucv = beta*(1+r)* (diag(1./proj.lambda)Transpu*diag(proj.lambda)) *(xsiE.*proj.lambda.*Ucv)  + proj.nu(:) + proj.er(:);
% check    ZZ2 = 0


%%% projecting  welfare (eta)
eta                = (proj.ul(:)./proj.lambda)./U(cv,Lsp);

%%% recap
proj.method        = 2;             % 1 for explicit 2 for implicit
proj.lambdam       = proj.lambdam ; % matrix form
proj.lambda        = proj.lambda ;  % vector form
proj.c             = proj.c;
proj.a             = proj.a;
proj.ap            = proj.ap;
proj.w             = proj.w;
proj.nu            = proj.nu;
proj.v             = proj.v;
proj.uc            = proj.uc;
proj.ul            = proj.ul;
proj.er            = proj.er; 
proj.ind           = proj.ind;

%%%  projection result
proj.Nbin           = Nbin;
proj.xsiE           = xsiE;
proj.Transp         = Transp;    % backward agents
proj.N              = N;
proj.Ncons          = Ncons;
proj.Transpa        = Transpa;   % backwar wealth
proj.eta            = eta;
proj.ls             = lle;
proj.Transpu        = Transpu;   % forward Euler  per capita forward




%%% to follow parameters
proj.r              = bewley.r;
proj.beta           = bewley.bet;
proj.alpha          = bewley.alpha;
proj.delta          = bewley.delta;
proj.wage           = bewley.wage;
proj.tau            = bewley.tau;
proj.phi            = bewley.b;
proj.sigma          = bewley.sigma;
proj.nx             = bewley.nx;     
proj.ne             = bewley.ne;
proj.ex             = bewley.ex;
proj.coefa          = coefa;
proj.v              = proj.v(:)./proj.lambda; %per capita
proj.cv             = cv;
proj.Ucv            = Ucv;
proj.Lsp            = Lsp;

proj.vareta         = var(proj.lambda.*proj.eta);
proj.varxsiE        = var(proj.lambda.*proj.xsiE);
proj.meaneta        = mean(proj.eta);
proj.meanxsiE       = mean(proj.xsiE);

proj.sdtdE          = std(proj.xsiE);
proj.sdtdW          = std(proj.eta);

for i = find(proj.lambda==0)
    proj.c(i)       = 0;
    proj.a(i)       = 0;
    proj.ap(i)      = 0;
    proj.w(i)       = 0;
    proj.nu(i)      = 0;
    proj.v(i)       = 0;
    proj.uc(i)      = 0;
    proj.ul(i)      = 0;
    proj.cv(i)      = 0;
    proj.Ucv(i)     = 0;
    proj.xsiE(i)    = 0;
    proj.eta(i)     = 0;
end;



