function [params,distr] = solve_steady(params, r0, maxiter,maxV,display)

%%%%%%%%%%%%%%%%%%%%%%%
beta            = params.bet;
alpha           = params.alpha;
xsil            = params.xsil;
phil            = params.phil;
Urec            = bewley.Urec;

Ntot            = bewley.Ntot;
% utility functions and related

Uc              = @(C,l) (C - (l.^(1+1/phil))/(xsil*(1+1/phil))).^(-bewley.sigma);
if bewley.sigma == 1
    U           = @(C,l) log(C - (l.^(1+1/phil))/(xsil*(1+1/phil)));
else
    U           = @(C,l) (((C - (l.^(1+1/phil))/(xsil*(1+1/phil)))).^(1-bewley.sigma))/(1-bewley.sigma);
end

%production function
F               = @(K,L) K.^(bewley.alpha) .* L.^(1-bewley.alpha) - bewley.delta * K;
FK              = @(K,L) bewley.alpha * K.^(bewley.alpha-1) .* L.^(1-bewley.alpha)  - bewley.delta;
FL              = @(K,L) (1-bewley.alpha)*K.^(bewley.alpha) .* L.^(-bewley.alpha);
wagefun         = @(x) (1-bewley.alpha)*((x+bewley.delta)/bewley.alpha).^(bewley.alpha/(bewley.alpha-1));


kgrid       = bewley.kgrid;
Trans       = kron(bewley.pie,bewley.piex); 
Le          = kron([0;1],kron(bewley.ex,ones(bewley.knum,1)));
Lu          = kron([1;0],kron(bewley.ex,ones(bewley.knum,1)));
v           = zeros(bewley.knum*bewley.ne*bewley.nx,1); 
lambda      = zeros(bewley.knum*bewley.ne*bewley.nx,1)/(bewley.knum*bewley.ne*bewley.nx);
lambda(1,1) = 1;

bewley.tau  = (1/(1-bewley.x))*Urec*bewley.b/(1 - Urec);
Transe      = [bewley.pie(1,1)*speye(Ntot/2),bewley.pie(2,1)*speye(Ntot/2);bewley.pie(1,2)*speye(Ntot/2),bewley.pie(2,2)*speye(Ntot/2)];

Transy      = kron(speye(bewley.ne),bewley.piex);     % bewley.nex*bewley.ne matrix change column but only productivity
Transy      = kron(Transy,speye(bewley.knum));
Transy      = Transy';
aa          = kron(ones(bewley.nx*bewley.ne,1),bewley.kgrid);

%%%%%%%%%%%%
r           =  r0; 
distr       = 10;

liter       = 1;               %Index for iteration on interest rate
gg          = 0.01;            % relaxation parameter

while  (distr>10^-5)&&(liter <= maxiter)
    wage    = wagefun(r);          %update of the wage value
    Util    = -10000*ones(bewley.knum*bewley.nx*bewley.ne,bewley.knum); % initialization of the period utility
    
    for i   = 1:bewley.knum       % past saving choice
        for j = 1:bewley.knum   % current saving choice
            for y =1:bewley.nx  % productivity 
                le = (xsil*(1-bewley.tau)*bewley.ex(y)*wage)^phil;
                    cu = kgrid(i)*(1+r) - kgrid(j) + bewley.b*wage*bewley.ex(y)*le;   
                    if (cu - ((bewley.b*le)^(1+1/phil))/(xsil*(1+1/phil)))>1e-6       % earn b * time the gross income of employed same prod.
                       Util(i+(y-1)*bewley.knum, j) = U(cu,bewley.b*le);
                    end;                    
                    ce = kgrid(i)*(1+r) - kgrid(j) + (1-bewley.tau)*wage*bewley.ex(y)*le;
                    if (ce - (le^(1+1/phil))/(xsil*(1+1/phil)))>1e-6
                        Util(i+(y-1 + bewley.nx)*bewley.knum, j) = U(ce,le);
                    end
            end
        end
    end
  
Iutil        = zeros(bewley.knum*bewley.ne*bewley.nx, bewley.knum); % intertemporal utility
decis       = zeros(bewley.knum*bewley.ne*bewley.nx,1);
test1       = 10;
test2       = 10;
cont        = 0;
      
 while ((test1 ~= 0 ) || (test2 > .00001)) && (cont<maxV)
    vt      = reshape(v,bewley.knum,bewley.nx*bewley.ne);   % row : asset, column type
    contv   = (Trans*vt');                                  % row : next period asset, colum type
    contt   = kron( contv,ones(bewley.knum,1));             %I repeat the continutation value to have the same size as a function of current asset
    Iutil   =  Util + beta*contt;                           % Intertemporal welfare has this simple expression
    %Taking the maximum         
    [res,tdecis] = max(Iutil');                             %row choice variable, column current asset*type
    tv           = res';
    test1 = max(max(abs(tdecis-decis)));
    test2 = max(max(abs(v-tv)));

    v     = tv;
    decis = tdecis;                                       
    cont  = cont+1;
    
 end
    
if (cont >= maxV)
        warning('Binding constraint on cont in loop')
end

%to have easy-to-understand matrix%j+(ef-1)*knum
deciskpv = 1 + mod(decis-1,bewley.knum);
deciskp  = reshape(deciskpv,bewley.knum,bewley.nx*bewley.ne);

%%%%%%%%%%%%%%%%%%%%%%
%%%% stationary distribution
%%%%%%%%%%%%%%%%%%%%%%
Transk = sparse(bewley.knum*bewley.ne*bewley.nx,bewley.knum*bewley.ne*bewley.nx);
for ty = 1:bewley.nx*bewley.ne
    for j = 1:bewley.knum
        ind              = (ty-1)*bewley.knum+j;        % index in lambda
        indt             = (ty-1)*bewley.knum + deciskp(j,ty); % new capital for unchange type
        Transk(ind,indt) = 1;                           % permute rows only
    end;
end;
Transk = Transk';
Transg = Transe*Transy*Transk;

test = 1;
while test > 10^-10
   lambda1   = Transg*lambda;
   test      = max(abs(lambda1-lambda));
   lambda    = lambda1;
end;

meanA        = aa'*lambda;

lle    = (xsil*(1-bewley.tau)*bewley.ex*wage).^phil;
meanL       = (1 -Urec)*bewley.exinv*(bewley.ex.*lle);  
rnew         =  FK(meanA,meanL);
distr2       = abs(r-rnew);                         % I iterate over the interest paid by huseholds

if distr2>distr
   gg        =  gg/2;
end;
distr        = distr2;
if maxiter>1
 r           = gg*rnew+(1-gg)*r;
end;

if display
    disp([ liter r*1000 meanA distr*1000 cont]);
end;

resv(liter,1) = r;
resv(liter,2) = rnew;
resv(liter,3) = meanA;
liter         = liter+1;

end;


%% reshaping the allocation in a matrice form and computing welfare


lambdam         = reshape(lambda,bewley.knum,bewley.nx*bewley.ne);
amat            = kron(kgrid,ones(1,bewley.nx*bewley.ne));  %beginning of period wealth
apmat           = kgrid(deciskp)                            % end-of-period wealth
wmat            = kron(ones(bewley.knum,1),[bewley.b*wage*(bewley.ex.*lle);(1-bewley.tau)*wage*(bewley.ex.*lle)]'); % lab income
cmat            = (1+r)*amat + wmat - apmat;

Alambdam        = sum(lambdam);

Ctot            = sum(sum(lambdam.*cmat));
Le              = kron(lle,ones(bewley.knum,1));
Lv              = [bewley.b*Le;Le];

% constructing the vector nu of Lagrange multiplier on the CC
nuv             = zeros(length(lambda),1);
indc            = find(deciskpv==1);
VV              = Uc(cmat(:),Lv)-beta*(1+r)*Transg'*Uc(cmat(:),Lv);
nuv(indc)       = VV(indc);
nuvm            = reshape(nuv,bewley.knum,bewley.nx*bewley.ne);


bewley.welf = sum(lambda(:).*v(:));

bewley.Transg   = Transg;
bewley.Transyk  = Transy*Transk;                            %backward (to construct B tilde)
bewley.decis    = decis;
bewley.lambda   = lambda;
bewley.U        = Urec;
bewley.tau      = bewley.tau;
bewley.meanA    = meanA;
bewley.meanL    = meanL;
bewley.v        = v;                                        % here it is per capita
bewley.r        = r;
bewley.wage     = wage;
bewley.c        = cmat(:);
bewley.Ctot     = Ctot;
bewley.nuvm     = nuvm;








